<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <meta http-equiv="X-UA-Compatible" content="ie=edge">
        <title>{% block title %}{{SITE_TITLE}}{% endblock %}</title>
        {% block head %}
        {% for content in head %}
        {% include content %}
        {% endfor %}
        {% endblock %}
    </head>
    <body {% if body_class is defined %}class="{{"".join(body_class)}}"{%endif%}>
        {% block header %}{% endblock %}
        {% block page_title %}{% endblock %}
        {% block content %}{% endblock %}
        {% block footer %}
        {% endblock %}
    </body>
</html>
